      program VECTOR  
*  Use in debugging on sun/4
*#include <f77/f77_floatingpoint.h>
c
c  Main program for plotting vector fields.
c  Program runs under a simple interpretive language with commands
c  like
c    file topography
c    read 43 55
c  See vector.doc for a complete description.
c  The program generates a PostScript file for plotting.
c  It is designed to be used as a module in a plotting script
c  which includes a color or contour plot file beneath the
c  vector field.  These other programs are used to properly
c  annotate the resulting plot -- this program only plots
c  arrows and an arrow scale vector.
c
c  Language interpretation is handled by the routines  peruse,
c  getval,  getchr.  Peruse reads in all commands up to certain
c  key points (read or plot), saving the list in a character array
c  for subsequent analysis by getval and getchr.  These routines
c  are called whenever values or strings are needed at the appropriate
c  point in the program.
c
      parameter (inmx=200)
      character *80 input(inmx), output
      common /dicta/ input
      common /outfil/ output
      common /ndict/ iecho,nin,istate(inmx),lin
c
      parameter (maxa=20 000)
      common /user/ a(4,maxa)

	  common /axes/  ax(4)
      common /fleche/ iakolor, iatype
c
      common /xtreme/ bord,amin,amax,anull(2),alim(3),colim(2)
      common /inout/ inp,iout,inf,info
c
      parameter (levmx=50)
      common /noir/ scutum,kolors,art(4,levmx),kart(levmx)
c
      data ihead/1/ xalabel/0.0/
c
c
*     ieeer= ieee_handler('set','common',SIGFPE_ABORT)
*     iecho=1
c  
c  
      do 5000 k=1, inmx
c  Read commands up to 'plot', 'read' or 'quit'
        call peruse(ihead)
        call getone('verbose', verb, nf)
        if (nf .eq. 1) info=verb
c
c  Input the array
        call getmat(mvec)
        ihead=0
        if (input(nin)(1:4) .eq. 'read') goto 5000
c
c  Draw the vector field
        call major(mvec)
        ihead=1
 5000 continue
      end
c______________________________________________________________________
      blockdata start
c
      parameter (inmx=200)
      common /ndict/ iecho,nin,istate(inmx),lin
      common /xtreme/ bord,amin,amax,anull(2),alim(3),colim(2)
      common /inout/ inp,iout,inf,info
c
      data iecho/0/, nin/0/
c
      data anull /0,0/, alim/0,0,0/
c
c inp=std input; iout=std output; inf=array diskfile; info=print freq
      data inp/5/,       iout/6/,             inf/7/, info/1/
c
      end
c______________________________________________________________________
      subroutine quit
c$$$$ calls getone, system
c  Brings program to an orderly conclusion;
c  Send file to screen if requested.
c  CAUTION: "system" is a SUN routine to invoke a UNIX shell and
c           show is local window display utility.  
c           Users on other systems delete the line containing system
c           below
      character*80 output
      common /outfil/ output
      common /inout/ inp,iout,inf,info
c
      write(9, '(a)')'showpage'
      close (unit=9)
      call getone('show', dum, nshow)
c  CAUTION: Next line is system-dependent!
cc      if (nshow .ge. 0) call system('show < '//output)
      write(iout,'(a)')' ',' Normal termination'
      stop
      end
c______________________________________________________________
      subroutine mabel(val, len, label)
c$$$$ calls no other routines
c  For a real number  val  assumed to be a decimal fraction rounded to
c  6 significant figures (or fewer), makes an equivalent character
c  string  label  of minimal length  len.
      character*8 form,local*20,label*(*)
      double precision ten, v
      data ten/10.0d00/
c
c  Find the number of significant figures in val
c
      nexp=0.0
      if (val .ne. 0.0) nexp=int(500.01 + log10(abs(val))) - 500
      v=dint(abs(val)/ten**(nexp-5) + 0.5)
      nres=nexp - 5
      do 1100 ipow=0, 4
        if (dmod(dint(v/ten**ipow+ 0.5), ten) .ne. 0.0) goto 1110
        nres=nres + 1
 1100 continue
 1110 nfld=3 + max(nexp, -nres, nexp-nres)
c
c  Build a format for writing the string
c
      ndpt=max(0, -nres)
      write(form, '( 2h(f, i2, 1h., i1, 1h))') nfld,ndpt
      if (ndpt .eq. 0) nfld=nfld - 1
c  If format is screwed up try a g format:
      if (index(form,'*') .ne. 0) form='(g12.4)'
      form='(1pe9.1)'
      nfld = 9
c
c  Construct a character string of minimal length for the label
c
      write(local, form) val
c  Remove leading blanks.
      is=1
      do 1200 i=1, nfld
        if (local(i:i) .eq. ' ') is=1 + is
 1200 continue 
      label=local(is:nfld)
      len=nfld - is + 1
      return
      end
c______________________________________________________________
      subroutine halt(messag)
c$$$$ calls no other routines
c  Program prints the message  messag  then stops.
      character*(*) messag
c
      print '(/1x, a)', messag
      stop
      end
c______________________________________________________________
      subroutine getmat(mvec)
c$$$$ calls inpvec
c  Routine for reading vector input a(4,*) 
c  where a(1)=x,a(2)=y,a(3)=u,a(4)=v.
c
c  mvec   size of vector array to be read.
c
      parameter (maxa=20 000)
      common /user/ a(4,maxa)
      common /inout/ inp,iout,inf,info
	  common /axes/  ax(4)
      character*75  file
      dimension val(37)
      save  file
      data  file(1:1)/' '/

c
c  Look for the axes statement in the command list
      call getval('axes', val, 4, nfound)
      if (nfound .eq. -99) then
        if((ax(1).eq.0.).and.(ax(2).eq.0.).and.(ax(3).eq.0.)) then
           write(iout,'(a)')
     $     '>>> You must invoke axes before reading vector data'
        endif
        return
      elseif (nfound .le. 3) then
        write(iout, '(a)')' ',
     $  '>>> axes must be followed by 4 numbers'
        return
      else
        ax(1)=val(1)
        ax(2)=val(2)
        ax(3)=val(3)
        ax(4)=val(4)
      if (info .ge. 1) write(iout,'(/(a,1x,1p4g12.3))')
     $' plot axes are   ',ax(1),ax(2),ax(3),ax(4)
      endif

c
c  Skip if axes are improperly specified to be equal.
      if (ax(1) .eq. ax(2) .or. ax(3) .eq. ax(4)) then
        write(iout,'(a)')
     $  '>>> x or y axes cannot have same start and end values'
        return
      endif

c
c  Look for the read statement in the command list
      call getval('read', val, 1, nfound)
      if (nfound .eq. -99) then
        if (mvec .gt. 0) return
        write(iout, '(a)')' ',
     $  '>>> No read statement in list. Provide one'
        mvec=0
        return
      elseif (nfound .le. 0) then
        write(iout, '(a)')' ',
     $  '>>> read must be followed by 1 number'
        mvec=0
        return
      else
        mvec=val(1)
      endif
      if (info .ge. 1) write(iout,'(/(a,1x,i12))')
     $' length of vector input mvec is   ',mvec

c
c  Impossible array dimensions trapped
      if (mvec.le.0) then
        write(iout, '(/a,i5/a)')
     $  '>>> Improper vector size dimensions',mvec,
     $  'Array cannot be accepted'
        if (abs(4*mvec).gt.maxa) write(iout,'(a,i8)')
     $  '>>> Total array size exceeds maximum permitted, ie ',maxa
        mvec=0
        return
      endif
c
c  Ascertain file name from file command
      call getchr('file', file, nf)
      if (file(1:1) .eq. ' ') then
        write(iout,'(a)')' ',
     $  '>>> No file name has been provided for data'
        mvec=0
        return
      endif
c
c  Open the file
      if (file .eq. '*') then
        inf=inp
      else
        inf=8
        open (unit=8, file=file,
     $     status='OLD', err=2000)
      endif
c
c  Now read the thing in
c
      call inpvec(mvec)

      if (inf .ne. inp) close (unit=8)
c
      return
c
c  Can't open the file for some reason.
 2000 write(iout,'(/a)')'>>> Nonexistent file or read permission denied'
      mvec=0
      return
      end
c______________________________________________________________
      subroutine inpvec(mvec)
c$$$$ calls getval
c  Routine actually to read formatted vector diskfile with * format.
c  It is assumed the file was written by rows with a statement like
c
c  write(iunit,format) (x(j),y(j),u(j),v(j), j=1, mvec)
c
      common /inout/ inp,iout,inf,info
      parameter (maxa=20 000)
      common /user/ a(4,maxa)

      character*1 dummyline
      dimension val(37)
c
c
c  Look for the read statement in the command list
      call getval('skip', val, 1, nfound)
      if (nfound .eq. -99) then
c...no skip statement, march onward
        go to 1000
      elseif (nfound .le. 0) then
        write(iout, '(a)')' ',
     $  '>>> skip must be followed by 1 number, but continuing read...'
      else
        mskip=val(1)
      endif

      do 500 i=1,mskip
  	     read(inf,'(a)',err=2010,end=2000) dummyline
 500  continue


 1000 continue
      do 10 i=1,mvec
 10   read (inf,*,err=2010,end=2000) a(1,i),a(2,i),a(3,i),a(4,i)
c
      return
c
c  Error section.  Improper dimensions have been supplied
c
 2000 write(iout,'(a)') ' ','>>> Unexpected endfile encountered',
     $'Improper array size has been given -  File rewound'
      m=0
      if (inf.ne.inp) rewind inf
      return
c
 2010 write(iout,'(a)') ' ','>>> Error while reading array data',
     $'File rewound'
      m=0
      if (inf.ne.inp) rewind inf
      return
      end
c______________________________________________________________
      subroutine major(mvec)
c$$$$ calls  getchr, getval, glevel, letter, minmax, arrow
c
c  The major routine for producing color map of the array  a.
c  m, n  are the internal array dimensions.
c  mdim  is the row dimension as  a  is store (=m+2)
c
c  The routine positions the plot on the paper, opens the plotfile,
c  invokes the painting routine and positions the origin.
c  Postscript prolog written here also
c
      parameter (levmx=50)
      parameter (maxa=20 000)
      common /user/ a(4,maxa)

      dimension xyorg(2),xynew(2)
      common /xtreme/ bord,amin,amax,anull(2),alim(3),colim(2)
      parameter (lpmx=2500, knmx=50 000)
      common /trace/ level,em,en,xfac,yfac
      common /noir/ scutum,kolors,art(4,levmx),kart(levmx)
      common /pallet/ many,npal(10),hue(28,10),sat(28,10),bri(28,10)
      common /inout/ inp,iout,inf,info
      common /local/ color(1),black(2)
      common /outfil/ output
      common /axes/  ax(4)
      common /fleche/ iakolor, iatype

c
      logical  doalabel

      dimension hw(3),val(37)
      character*80 output, label, onoff*2, anote
      save kase, hw, xyorg, xynew
      data kase/0/,  hw/2*0, 0.08333/, nlev/0/
      data xyorg/0,1000/, xynew/0,0/
      data onoff/'of'/
c
c  No array has been provided. Abandon ship
c
      if (mvec.eq. 0) then
         write(iout, '(/a)')'>>> Nothing to plot - read an array'
         return
      endif
c
c  Get a substitute color table for palette 1 if desired
      call table
c
      kase=1 + kase
      loops=0
      if (kase .eq. 1) then
        output='myvect'
        call getchr('output', output, nf)
        open(unit=9, file=output)
        write(9,'(a)') '%!','initmatrix',
     $  '0.072 0.072 scale',
     $  '200 200 translate',
     $  '/m {newpath moveto} def','/d {rlineto} def',
     $  '/u {rmoveto} def','/l {lineto} def'

        write(9,'(a)') 
     $  '/arrow0 { newpath 1 0 moveto ',
     $  ' 0 -0.2 lineto ',
     $  ' 0  0.2 lineto closepath fill   } def'

        write(9,'(a)') 
     $  '/arrow1 { newpath 0.8 0 moveto ',
     $  ' 0 0 lineto  stroke',
     $  ' 1  0 moveto ',
     $  ' 0.7 -0.1 lineto ',
     $  ' 0.7  0.1 lineto closepath fill   } def'

        write(9,'(a)') 
     $  '/arrow2 { newpath 1.0 0 moveto ',
     $  ' 0 0 lineto  stroke   } def'
      endif
c 
c  Choose available colors from a standard pallet
c
      call getval('pallet', color, 3, nblck)
      if (nblck.gt.0 .and. kolors.eq.0) then
          kolors=10
          match=color(1)
          if(match.gt.0) kolors = npal(match)
          if (info.ge.1) write(iout,'(a,i3)')
     $    ' Colors drawn from standard pallet ',match
c  If this is a greytone (match=0) get shade parameters
          if (match .le. 0) then
            do 1600 k=1, kolors
              art(1,k)=k
              art(2,k)=0.0
              art(3,k)=0.0
              art(4,k)=(1.0-((k-1)/(kolors-1)))
 1600       continue
           else
c  Canned pallet hsb vales obtained from common /artist/
c
            do 1700 k=1, kolors
              art(1,k)= k
              art(2,k)=hue(k,match)
              art(3,k)=sat(k,match)
              art(4,k)=bri(k,match)
 1700       continue
          endif
c
        if (info .ge. 1) then
          write(iout,'(a)')' ',
     $ '  These levels and color codes have been set',
     $ '     level          hue   saturation brightness'
         do 1800 kl=1, kolors
           k=kart(kl)
           write(iout,'(i2,g12.4,3f10.3)') kl, (art(j,k),j=1,4)
 1800    continue
        endif
      endif
c
c   Determine extremes of function to be plotted; if interval has
c   been set explicitly retain the values.
      call minmax(mvec, a, vmin, vmax)
      if (info .ge. 1) write(iout,'(/(a,1p,2g12.4))')
     $' Vector magnitudes fall within   ',vmin,vmax
c
c  Decide on the size of the map; it is proportional to the axis dimensions
c  unless otherwise requested
      call getval('height', hw(2), 1, nf)
      call getval('width',  hw(1), 1, nf)
      w=hw(1)
      h=hw(2)
      if (h .le. 0 .and. w .eq. 0) then
         w=6.0
         h=6.0
      endif
      if (w .le. 0) w=h*abs(ax(2)-ax(1))/abs(ax(4)-ax(3))
      if (h .le. 0) h=w*abs(ax(4)-ax(3))/abs(ax(2)-ax(1))
c
c  Set the origin.
c
c  If the origin of this plot is explicitly set, move it.
c  External units are 1/1000 inch for compatibility with pscrip.
c  Program units are inches.
      c=1000
c
      call getval('plot', xyorg, 2, nf)
      if (nf .eq. 2) then
        write(9,'(2f9.2,a)') 
     $  c*xyorg(1), c*xyorg(2), ' translate'
        xynew(1)=w
        xynew(2)=-xyorg(2)
      else
c  No explicit origin requested.  Set standard position.
        write(9,'(2f9.2,a)') 
     $  c*xynew(1)+c, c*xynew(2)+c,' translate'
        xynew(1)=w
        xynew(2)=-1.0
      endif
c
c  Plot label if there is one.
      dy=0.0
      call getchr('label', label, nf)
c
      hlet=int(0.5*h + 9.0)/72.0
      if (nf .eq. 1) then
        if (info .ge. 1) write(iout,'(/2a)') ' Label: ',label(1:72)
        dy=dy - 2.0*hlet
        call letter(0.0, dy, hlet, 0.0, label)
      endif
c
c  Plot arrow label if there is one                
c
      call getval('alabel', value, 1, nf)
c
      if(nf .eq. -99) then
         doalabel = .FALSE.
      else
         doalabel = .TRUE.
      endif

      if (nf .eq. 1) then
        asize = value
      else
        asize = vmax
      endif

      if(doalabel) then

        dy=0.0
        hlet=int(0.5*h + 9.0)/72.0
        dy=dy - 6.0*hlet

        xalabel = 1.5*(kase-1)
        if(nf.eq.0) then
          call mabel(asize, klen, anote)
          call letter(0.2+xalabel, dy, hlet, 0.0, anote(1:klen))
        else
          call getlastchr( anote, nfound )
          if (nfound .eq. 1) then
            if (info .ge. 1) write(iout,'(/2a)') ' alabel: ',anote(1:72)
            call letter(0.2+xalabel, dy, hlet, 0.0, anote)
          endif
        endif
      endif
c
c  Before plotting arrow in arrow label, set arrow color 
c  for this and next suite of arrows
c
      call getval('arro', val, 2, nf)
      if (nf .eq. -99) then
c...use default arrow style, color
        iatype = 0
        iakolor = 0
      else if (nf .eq. 2) then
        iatype = val(1)
        iakolor = val(2)
      else  if (nf .eq. 1) then
c...use default color (black)
        iatype = val(1)
        iakolor = 0
      endif
c
c
 222  continue
      if(iakolor.ge.1) then
        call newpn(iakolor)
c the following bit is jason's version
c       if(kolors.eq.0) then
c... if choose colors without pallet, print warning, then 
c    set arrow color to black and proceed along
c          write(iout,*) 
c    $    'must choose pallet before selecting nongray color'
c         iakolor = 0
c         go to 222
c       endif

c
c       if(iakolor.gt.kolors) then
c          write(iout,*) 'iakolor>max kolors, set to max kolors'
c          iakolor = kolors
c       endif

c       ka=kart(iakolor)
c       write(9,'(3f10.3,a)')
c    $  art(2,ka),art(3,ka),art(4,ka),' sethsbcolor '

      else if(iakolor.eq.0) then
        write(9,'(a)') ' 0 setgray '
      else if(iakolor.eq.-1) then
        write(9,'(a)') ' 1 setgray '
      else if(iakolor.eq.-2) then
        write(9,'(a)') ' 0.75 setgray '
      else if(iakolor.eq.-3) then
        write(9,'(a)') ' 0.5 setgray '
      else if(iakolor.eq.-4) then
        write(9,'(a)') ' 0.25 setgray '
      else 
        write(9,'(a)') ' 0 setgray '
        write(iout,'(a)') 'illegal arrow kolor chosen, set to black'
      endif
c
      call arrow(xalabel,dy,0.,(dy+abs(min(w,h)/(20.))))
c
c  Plot the arrows 
c
      if (info .ge. 1) write(iout,'(/a)') ' Plotting underway ...'
c
      call plotit(mvec, w, h, asize)
c
      return
      end
c______________________________________________________________
      subroutine plotit(mvec, w, h, asize)
c$$$$ calls arrow
c  Does basic arrow plotting.
c  As always,  a is the vector array
c  h, w  are height and width in inches of finished map.
      dimension basx(4),basy(4)
      dimension val(37)
      parameter (maxa=20 000)
      common /user/ a(4,maxa)

      common /trace/ level,em,en,xfac,yfac
      common /xtreme/ bord,amin,amax,anull(2),alim(3),colim(2)
	  common /axes/  ax(4)
c
      data basx/4*0/, basy/4*1/

c
c  Set scale factors which are single grid steps in x and y
      c=1000
      xfac=c*w/(abs(ax(2)-ax(1)))
      yfac=c*h/(abs(ax(4)-ax(3)))
      ascale = min(w,h)/(20.*asize)

c
c  Read in sift or interleave factor to skip over sift vectors between each
c  plotted vector

      nsift = 1
      msift = nsift
c
c  Look for the decimate statement in the command list
      call getval('sift', val, 1, nfound)
      if (nfound .eq. -99) then
        write(iout,*) 'no sift statement found'
c...no sift statement, march onward
        go to 1000
      elseif (nfound .le. 0) then
        write(iout, '(a)')' ',
     $  '>>> sift must be followed by 1 number, but continuing read...'
      else
        msift= nsift + (val(1) + 0.00001)
      endif
 1000 continue
c
c  Plot vector data                          


        do 30 i=1, mvec, msift           

          x0 = w*(a(1,i)- ax(1))/(ax(2)-ax(1))
          y0 = h*(a(2,i)- ax(3))/(ax(4)-ax(3))
          x1 = x0 + ascale*a(3,i)
          y1 = y0 + ascale*a(4,i)
          call arrow(x0,y0,x1,y1)
 30     continue
 
      return
      end
c____________________________________________________________________
      function arctan(y,x)
c------------------------------------------------------------
c       version of arctan which will not choke on (0,0) entry
c------------------------------------------------------------
      pi  =  3.1415926535898
      pi2 =  1.5707963267949
      if(x) 2,1,2
 1    arctan = sign(pi2,y)
      return
 2    arctan = atan(y/x)
      if(x) 3,3,4
 3    arctan = arctan + sign(pi,y)
 4    return
      end
c____________________________________________________________________
      subroutine arrow(x0,y0,x1,y1)
c...draws an arrow connecting plot points (x0,y0) to (x1,y1)
c...using an arrow style stored in nib and color kolor
cc      write(*,*) x0,y0,x1,y1
      common /fleche/ iakolor, iatype


      c = 1000.

      dx = x1 - x0
      dy = y1 - y0
      atheta = arctan( dy, dx )
      dtheta = 180.*atheta/3.141592654
                                                               
      alength = sqrt(dx**2 + dy**2)

      IF(c*alength.gt.1.e-4) THEN

      if(iatype.eq.0) then
c...should be default triangular wedge arrow

         write(9,'(a,2f12.4,a,f12.4,a,2f12.4,a)') ' gsave ',
     $   c*x0,c*y0,' translate ',
     $   dtheta, ' rotate ',
     $   c*alength, c*alength, ' scale arrow0 grestore ' 

      else if(iatype.eq.1) then
c...should be standard arrow (check postscript init section)

         write(9,'(a,2f12.4,a,f12.4,a,2f12.4,a,f12.4,a)') ' gsave ',
     $   c*x0,c*y0,' translate ',
     $   dtheta, ' rotate ',
     $   c*alength, c*alength, ' scale ',
     $   5.0/(c*alength), ' setlinewidth arrow1 grestore ' 

      else if(iatype.eq.2) then
c...just show vector dir., not magnitude on a triangular wedge arrow

         alength = 0.2
         write(9,'(a,2f12.4,a,f12.4,a,2f12.4,a)') ' gsave ',
     $   c*x0,c*y0,' translate ',
     $   dtheta, ' rotate ',
     $   c*alength, c*alength, ' scale arrow0 grestore ' 

      else if(iatype.eq.3) then
c...show  line segment (no arrow heads)
         write(9,'(a,2f12.4,a,f12.4,a,2f12.4,a,f12.4,a)') ' gsave ',
     $   c*x0,c*y0,' translate ',
     $   dtheta, ' rotate ',
     $   c*alength, c*alength, ' scale ',
     $   5.0/(c), ' setlinewidth arrow2 grestore ' 

      endif
      ENDIF

      return
      end
C
C______________________________________________________________
      		
	   subroutine plot(x,y,kpen)
C	Absolute draw: kpen = 2, Absolute move: kpen = 3
       c = 1000.
       nx = int(x*c)
       ny = int(y*c)
	   if (kpen .eq. 2) then
	     write(9,'(2i10,a)') nx,ny,' l '
       else if(kpen.eq.3) then
	     write(9,'(2i10,a)') nx,ny, ' m '
	   endif
	   return
	   end
c_______________________________________________________________
      subroutine table
c$$$ calls getchr
c  Reads in a palette from a file, substituting it for
c  the standard palette number 1.  Always reads to EOF
c
      parameter (nk=28)
      character*64 name
      common /inout/ inp,iout,inf,info
      common /pallet/ many,npal(10),hue(nk,10),sat(nk,10),bri(nk,10)
c
c  Look for filename of a color table
      call getchr('table', name, ntb)
      if (ntb .le. 0) return
      open (unit=10, file=name, status='OLD', err=2000)
c
c  Read into the first palette
      do 1100 it=1, nk
        read (10,*,end=1110,err=2100) hue(it,1),sat(it,1),bri(it,1)
 1100 continue 
      it=nk + 1
 1110 it=it - 1
      npal(1)=it
      write(iout, '(/a,i3,a)')
     $' Palette 1 has been replaced by a new selection of ',it,' colors'
      return
c
c  Errors reported
 2000 write(iout, '(a,a/a)')'>>> File ',name,
     $'is nonexistent or you lack permission to read it'
      return
 2100 write(iout, '(2a)')'>>> Unreadable numbers in file ', name,
     $'Palette 1 will be a mixture of old and new colors'
      return
      end
c_______________________________________________________________________
        
      blockdata artist
c  Sets the color attributes of the various pallets and default color
c  schemes.  Setting the variables in /pallet/ is the main task.
c
      common /local/ color(1),black(2)
c
      parameter (levmx=50)
      common /noir/ scutum,kolors,art(4,levmx),kart(levmx)
c
      common /pallet/ many,npal(10),hue(28,10),sat(28,10),bri(28,10)
c
      data color/0/,  black/ 0.7, 2.0/
      data kolors/0/, art/levmx*1.0,levmx*1.0,levmx*1.0,levmx*1.0/
c
      data many/5/
c
c  Pallet 1:  low cool blue to high hot red (More colors than 1)
      data npal(1)/16 /
      data (hue(j,1),sat(j,1),bri(j,1), j=1, 16)/
     $0.600,1.000,0.500, 0.600,1.000,0.600, 0.600,1.000,0.700,
     $0.600,1.000,1.000, 0.600,0.800,1.000, 0.600,0.700,1.000,
     $0.600,0.500,1.000, 0.600,0.300,1.000, 1.000,0.100,1.000,
     $1.000,0.300,1.000, 1.000,0.350,1.000, 0.900,0.500,1.000,
     $1.000,0.550,1.000, 1.000,0.600,1.000, 1.000,1.000,1.000,
     $1.000,0.500,0.800/
c
c  Pallet 2:  blue-green-yellow-orange-brown-red-lilac
      data npal(2)/24 /
      data (hue(j,2),sat(j,2),bri(j,2), j=1, 24)/
     $0.600,1.000,0.500,0.600,1.000,0.700,0.600,1.000,1.000,
     $0.550,1.000,0.800,0.500,1.000,0.700,0.500,1.000,0.800,
     $0.450,1.000,0.800,0.400,1.000,0.800,0.400,1.000,1.000,
     $0.300,0.600,1.000,0.250,0.600,1.000,0.200,0.600,1.000,
     $0.150,0.300,1.000,0.100,0.400,1.000,0.100,0.800,1.000,
     $0.100,0.900,1.000,0.100,0.5,0.800,0.100,1.000,0.800,
     $0.000,1.000,0.900,0.000,0.5,0.900,0.000,0.500,1.000,
     $0.000,0.400,1.000,0.000,0.300,1.000,0.000,0.200,1.000/
c
c  Pallet 3: lilac-blue-green-yellow-orange (pastel)
      data npal(3)/28/
      data (hue(j,3),sat(j,3),bri(j,3), j=1, 28)/
     $0.00,0.00,0.90, 0.90,0.30,1.00, 0.80,0.30,0.90, 0.80,0.40,0.90,
     $0.80,0.60,1.00, 0.70,0.40,1.00, 0.70,0.30,1.00, 0.60,0.30,1.00,
     $0.48,0.30,1.00, 0.48,0.40,1.00, 0.40,0.50,1.00, 0.47,1.00,1.00,
     $0.40,0.70,1.00, 0.30,0.50,1.00, 0.20,1.00,1.00, 0.20,0.80,1.00,
     $0.20,0.40,1.00, 0.20,0.20,1.00, 0.00,0.20,1.00, 0.00,0.25,1.00,
     $0.00,0.35,1.00, 0.00,0.40,1.00, 0.00,0.45,1.00, 0.00,0.55,1.00,
     $0.00,0.60,1.00, 0.00,0.90,1.00, 0.00,1.00,0.80, 0.00,1.00,0.70/
c
c  Pallet 4: the Harvard pallet dark blue-light blue-yellow-brown
      data npal(4)/24/
      data (hue(j,4),sat(j,4),bri(j,4), j=1, 24)/
     $ 0.60,1.00,0.40, 0.60,0.90,0.50, 0.60,0.70,0.50, 0.60,1.00,0.81,
     $ 0.60,0.70,0.70, 0.60,1.00,1.00, 0.60,0.75,1.00, 0.60,0.70,1.00,
     $ 0.60,0.55,1.00, 0.60,0.40,1.00, 0.60,0.30,1.00, 0.55,0.25,1.00,
     $ 0.1,.4,1, 0.1,0.4,0.9, 0.1,.8,1, 0.10,0.90,1.00,
     $ 0.10,1.00,0.90, 0.00,0.60,1.00, 0.00,0.85,1.00, 0.00,0.50,0.80,
     $ 0.00,1.00,0.70, 0.0,.5,.5, 0.00,1.00,0.60, 0.00,0.50,0.40/
c
c  Pallet 5: another attempt at pallet 1
      data npal(5)/20/
      data (hue(j,5),sat(j,5),bri(j,5), j=1, 20)/
     $ 0.60,1.00,0.40,0.60,0.70,0.50,0.60,1.00,0.81,0.60,0.70,0.70,
     $ 0.60,1.00,1.00,0.60,0.75,1.00,0.60,0.70,1.00,0.60,0.40,1.00,
     $ 0.60,0.30,1.00,0.55,0.25,1.00,0.90,0.15,1.00,0.90,0.25,1.00,
     $ 0.90,0.35,1.00,0.90,0.45,1.00,0.90,0.65,1.00,0.90,0.95,1.00,
     $ 1.00,1.00,1.00,1.00,0.50,0.70,1.00,1.00,0.70,1.00,1.00,0.50/
      end
c_______________________________________________________________
      subroutine peruse(ihead)
c$$$$ calls quit
c  The command input routine.
c  Reads from the standard input until eof or plot.
c  Saves the lines in the character array input for later
c  parsing by getchr and getval.
c  Prints a list of commands for this group of routines
      parameter (inmx=200)
      character *80 input(inmx),line
      common /dicta/ input
      common /ndict/ iecho,nin,istate(inmx),lin
      common /inout/ inp,iout,inf,info
c
c
      if (ihead .ne. 0) write(iout,'(a)') ' ',
     $' Enter Vector commands '
c
      do 1500 l=nin+1, inmx
        read(inp,'(80a)', end=2500) line
        if (line(1:1) .ne. ' ') then
          nin=nin + 1
          input(nin)=line
          istate(nin)=0
        endif
        if (line(1:4) .eq. 'hist') then
          nin=nin - 1
          write(iout,'(/a/(i3,a,a75))') 'Command history to date',
     $  (k,': ',input(k)(1:74), k=1, nin)
        endif
        if (line(1:4) .eq. 'plot') goto 2000
        if (line(1:4) .eq. 'read') goto 2000
        if (line(1:4).eq.'stop' .or. line(1:4).eq.'quit') call quit
 1500 continue
c
 2000 return
c
 2500 call quit
      end
c______________________________________________________________________
      subroutine getval(code, values, nwant, nfound)
c$$$$ calls no other routines
c
c  Evaluates numbers in the string  char.  nwant  numbers are expected,
c  nfound  are actually found in the character string.  If an error is
c  discovered  nfound=-n,  where  n  is the number of numbers properly
c  decoded.  If there are no numbers after the codeword nfound=0.
      parameter (inmx=200)
      character *80 input(inmx),line
      common /dicta/ input
      common /ndict/ iecho,nin,istate(inmx),lin
      common /inout/ inp,iout,inf,info
      dimension values(*)
      character bore(2)*1, form*8, code*4, char*80
c
      data bore/'e',' '/
c
ce
      do 1010 lin=nin, 1, -1
        line=input(lin)
        if (code .eq. line(1:4) .and. istate(lin) .eq. 0) then
          if (iecho .ge. 1) write(iout,'(2a)')'>>> ',line
          istate(lin)=istate(lin) + 1
          nb=index(line, ' ')+1
          char=line(nb:80)
          kn=80 - nb + 1
          goto 1020
        endif
 1010 continue
c  code word not found
      nfound=-99
      return
c
 1020 continue
      k1=1
c  Up to  nwant  numbers are sought.
      do 1800 nval=1, nwant
        do 1100 k=k1, kn
          if (char(k:k) .ne. ' ') goto 1200
 1100   continue
        nfound=nval-1
        return
 1200   iex=1
        do 1300 l=k, kn
          if (char(l:l).eq. ',') goto 1500
          if (char(l:l).ne. ' ') goto 1300
          if (char(l-1:l-1).ne.bore(iex)) goto 1500
          iex=2
 1300   continue
 1500   m=l - k
c  Augment unpredictable error trapping of some compilers e.g. Sun
        if (index('Ee+-.0123456789', char(k:k)) .eq. 0) goto 2000
        k2=l+1
        write(form, '(2h(f, i3, 3h.0) )') m
        read (char(k:l-1), form, err=2000) values(nval)
 1800 k1=k2
      nval=1 - nwant
 1900 nfound=1 - nval
      return
c
 2000 write(iout, '(a)')' ',
     $'***** Unreadable numbers in this input line:',line
      goto 1900
      end
c______________________________________________________________
      subroutine getlastchr( char, nfound)
c$$$$ calls no other routines
      parameter (inmx=200)
      character *80 input(inmx),blank, line, char*(*)
      common /dicta/ input
      common /ndict/ iecho,nin,istate(inmx),lin
      data blank/' '/
c
      line=input(lin)
      nb=index(line, ' ')+1
      do 1200 k=nb, 80
        if (line(k:k) .ne. ' ') then
          char=line(k:80)
          nfound=1
          return
        endif
 1200 continue
c
c  Blank field after code word
      nfound=0
      return
      end
c______________________________________________________________
      subroutine getchr(code, char, nfound)
c$$$$ calls no other routines
      parameter (inmx=200)
      character *80 input(inmx),blank, line, code*4, char*(*)
      common /dicta/ input
      common /ndict/ iecho,nin,istate(inmx),lin
      common /inout/ inp,iout,inf,info
      data blank/' '/
c
      do 1010 lin=nin, 1, -1
        line=input(lin)
        if (code .eq. line(1:4) .and. istate(lin) .eq. 0) then
          if (iecho .ge. 1) write(iout,'(2a)')'>>> ',line
          istate(lin)=istate(lin) + 1
          goto 1020
        endif
 1010 continue
c  code word not found
      nfound=-99
      return
c
 1020 continue
      nb=index(line, ' ')+1
      do 1200 k=nb, 80
        if (line(k:k) .ne. ' ') then
          char=line(k:80)
          nfound=1
          return
        endif
 1200 continue
c
c  Blank field after code word
      nfound=0
      return
      end
c______________________________________________________________
      subroutine minmax(mvec, x, xmin, xmax)
c$$$$ calls no other routines
c  Finds minimum and maximum of an array  x  with actual dimensions
c  n, k  held in columns that are mdim long
c  k.  
      dimension x(4,*)
      xmin=sqrt(x(3,1)**2 + x(4,1)**2)
      xmax=xmin
      do 1000 i=1, mvec
        xmin=min(xmin, sqrt(x(3,i)**2 + x(4,i)**2))
        xmax=max(xmax, sqrt(x(3,i)**2 + x(4,i)**2))
 1000 continue
      return
      end
c______________________________________________________________________
      subroutine getone(code, value, nfound)
c$$$$ calls getval
c
c  Evaluates a single number from the input store /dicta/ associated
c  with the keyword code.  The answer is returned in  value.
c  nfound  may be 1 if a number is read, 0 if the field after
c  code is blank, -99 if code is absent.
c  
      dimension values(1)
      character code*4
c
      call getval(code, values, 1, nfound)
      if (nfound .eq. 1) value=values(1)
      return
      end
C______________________________________________________________
      subroutine letter(x, y, h, angle, text)
c$$$$ calls no other routines
      character*(*) text
c  A postscript version of the famous routine, without Greek or
c  subscript ability.
c  External units are reset to points (1/72 inches);
c  Program units as always are inches.
      long=len(text)
      do 1100 l=long, 1, -1
        if (text(l:l) .ne. ' ') goto 1101
 1100 continue
      return
 1101 continue

cc      ipoint=h*72
      ipoint=2*h*72
       write(9,'(a)')'gsave 15.0 13.89 scale 0 setgray',
     $'/Times-Roman findfont'
c      write(9,'(a)')'gsave  4.5 4.1667 scale 0 setgray',
c      write(9,'(a)')'gsave   6.0 5.5555 scale 0 setgray',{400 dpi}
 
      write(9,'(i4,a/2f6.1,a)')
     $ipoint,' scalefont setfont',
     $67*x, 72*y, ' moveto currentpoint translate'
      if (angle .ne. 0) write(9,'(f8.2,a)') angle,' rotate'
c
      write(9,'(3a)') '(',text(1:l),') show'
c
      write(9,'(a)') 'grestore'
      return
      end
C
c=======================================================================
c_______________________________________________________________
      subroutine newpn(ipen)
c$$$$ calls no other routines
c  Uses Postscript command to reset the color and line width.
c  Color table held in array below as hsb values
      parameter (ncol=10)
      dimension table(3,ncol)
      save lastpn
      data lastpn/-9/
c
c 0 or 1: Black; 2: Red; 3: Blue; 4: Green; 5: Brown; 6: Orange;
c 7: Yellow; 8: Purple; 9:gray; 10 white
      data ((table(i,j),i=1,3), j=1, ncol)/
     $0,0,0,      0,1,1,     0.667,1,1,  0.333,1,1, 0.1,1,.7,
     $0.1,0.8,1,  0.15,1,1,  0.833,1,1,  0,0,0.4,   0,0,1/
c
c  Do not bother to change color if it doesn't change
      if (ipen .eq. lastpn) return
c  Flush current polygon buffer with previous parameters
      write(91,*) 'currentpoint k'
c  Ipen/1000 is interpreted as line width in 1/000 inches
      iwide=ipen/1000
      ip=min(max(mod(ipen, 1000), 1), ncol)
      if (iwide.gt.0) write(9,'(i5,a)') iwide,' setlinewidth'
      if (iwide.eq.0) write(9,'(i5,a)')     4,' setlinewidth'
c
c  By convention ipen=0, 1 => black
      write(9,'(3f8.3,a)') (table(i,ip),i=1,3),' sethsbcolor'
      lastpn=ipen
      return
      end
c_______________________________________________________________


