      parameter (mdat=300000,mmod=50000,mxmat=20000000)
      common/amat1/amat(mxmat)
      common/nmat1/nmat(mxmat)
      common/data/res(mdat),mmat(mdat)
      common/diwt/dwt(mmod)
      dimension ressv(mdat)
      dimension v(mmod),w(mmod),x(mmod)
      common/blocks/r1(30),r2(30),hsz(180),bsz,
     +    nshell,mshell,mlat(181),nlat
      character*256 filename
      character*4 nsta

c*** get block structure
      call blks(nblk)
      if(nblk.gt.mmod) then
        print *,'mmod needs increasing',mmod,nblk
        stop
      end if
      do i=1,nblk
         dwt(i)=0.
      enddo

c*** read in matrices
      ntot=0
      ib=0
      mmat(1)=1
      print *,'enter number of surface wave matrices : '
      read(*,*) matnum
      if(matnum.le.0) stop
      do imat=1,matnum
        nbase=ntot
        write(*,'(a,$)') 'enter matrix file name: '
        read(*,'(a256)') filename
        print *,'enter multiplier for errors'
        read(*,*) emult
        indx=0
        open(10,file=filename, form='unformatted')
   20   read(10,end=30) rem,reserr,nscan
        indx=indx+1
        reserr=reserr*emult
        ressv(indx+nbase)=rem/reserr
        if(mod(indx,1000).eq.0) print *,indx,ib+nscan
        read(10) (amat(i+ib),i=1,nscan)
        read(10) (nmat(i+ib),i=1,nscan)
        do i=1,nscan
           amat(i+ib)=amat(i+ib)/(reserr)
           mb=nmat(i+ib)
           dwt(mb)=dwt(mb)+amat(i+ib)**2
        enddo
        ib=ib+nscan
        mmat(nbase+indx+1)=ib+1
        goto 20
   30   ntot=ntot+indx
      enddo
      print *,'matrices read in'

c*** construct smoother
      flam=0.
      print *,'enter lambda in lateral smoother'
      read(*,*) glam
      call smooth(ntot,flam,glam)
      print *,'smoother constructed'
      do i=1,nblk
        if(dwt(i).gt.0.) then
            dwt(i)=1./sqrt(dwt(i))
        else
            dwt(i)=1.
        end if
      enddo

c*** now do the iterations!
      print *,'enter itmax'
      read(*,*) itmax
      print *,'enter number of realizations '
      read(*,*) nr
      m=ntot
      n=nblk
      print *,m,n,mmat(ntot+1)-1
      dum=0.
      open(9,file='tomo.err3',form='unformatted')
      write(9) n,dum
      iseed=-2
      do i=1,nr
c*** perturb data array -- we divided through by error on reading
c*** so noise is gaussian with unit variance
        do k=1,m
          res(k)=ressv(k)+gasdev(iseed)
        enddo
        call pstomo(m,n,x,res,v,w,itmax)
      enddo
      close(9)

      stop
      end   

      subroutine pstomo(m,n,x,u,v,w,itmax)
c*** Nolet's version of the LSQR algorithm
c input:
c  m = number of data
c  n = number of unknowns
c  u is the data vector which is overwritten
c  itmax is the number of iterations
c
c   v(n) and w(n) are scratch arrays
c output:
c  x = solution
c
c we are solving Ax=u
c avpu(m,n,u,v) computes u=u+Av
c bvpv(m,n,u,v) computes v=v+A(T) u
c
c  note that phibar and r are the absolute and relative
c  lengths of the residual vector Ax-u
c
      parameter (mmod=50000)
      common/diwt/dwt(mmod)
      dimension x(*),u(*),v(*),w(*),xdum(mmod)
      real*8 beta,alfa
      do i=1,n
        x(i)=0.
        v(i)=0.
      enddo
      sumlast=0.
      call normlz(m,u,beta)
      b1=beta
      call bvpv(m,n,u,v)
      call normlz(n,v,alfa)
      rhobar=alfa
      phibar=beta
      do i=1,n
        w(i)=v(i)
      enddo
      print *,' phibar is the absolute value of |Ax-b|'
      print *,' r is the relative value of |Ax-b|'
      print *,' it#   x(1)          phibar          r       '//
     +  '     |x|         chisq/n      npar'
      chisq=phibar*phibar/m
      print 900,0,0.,phibar,1.,0.,chisq
      do 50 iter=1,itmax
        a=-alfa
        do i=1,m
          u(i)=a*u(i)
        enddo
        call avpu(m,n,u,v)
        call normlz(m,u,beta)
        b=-beta
        do i=1,n
          v(i)=b*v(i)
        enddo
        call bvpv(m,n,u,v)
        call normlz(n,v,alfa)
        rho=sqrt(rhobar*rhobar+beta*beta)
        c=rhobar/rho
        s=beta/rho
        teta=s*alfa
        rhobar=-c*alfa
        phi=c*phibar
        phibar=s*phibar
        t1=phi/rho
        t2=-teta/rho
        do i=1,n
          x(i)=t1*w(i)+x(i)
          w(i)=t2*w(i)+v(i)
        enddo
        r=phibar/b1
        sum=0.
        nnz=0
        do i=1,n
          if(x(i).ne.0.) nnz=nnz+1
          xdum(i)=dwt(i)*x(i)
          sum=sum+xdum(i)*xdum(i)
        enddo
        chisq=phibar*phibar/m
        tol=abs((sum-sumlast)/sum)
        print 900,iter,x(1),phibar,r,sqrt(sum),chisq,nnz,tol
  900   format(i4,5g14.6,i6,3x,g14.6)
c**** use length of model vector to check for convergence
        if(tol.lt.1.e-5) goto 60
        sumlast=sum
   50   continue
   60 write(9) iter,(xdum(i),i=1,n),phibar,r
      return
      end

      subroutine normlz(n,x,s)
c compute eucl norm of x (in s) and normalize x
      dimension x(*)
      real*8 s
      s=0.d0
      do i=1,n
        s=s+x(i)*x(i)
      enddo
      s=sqrt(s)
      ss=1.d0/s
      do i=1,n
        x(i)=ss*x(i)
      enddo
      return
      end

      subroutine avpu(m,n,u,v)
c we are solving Ax=u
c avpu(m,n,u,v) computes u=u+Av
c*** n=# model params, m=# data
      parameter (mdat=300000,mmod=50000,mxmat=20000000)
      common/diwt/dwt(mmod)
      common/amat1/amat(mxmat)
      common/nmat1/nmat(mxmat)
      common/data/res(mdat),mmat(mdat)
      dimension u(*),v(*)
      real*8 s
      do i=1,m
        s=0.d0
        do j=mmat(i),mmat(i+1)-1
          s=s+amat(j)*v(nmat(j))*dwt(nmat(j))
        enddo
        u(i)=u(i)+s
      enddo
      return
      end
  
      subroutine bvpv(m,n,u,v)
c we are solving Ax=u
c bvpv(m,n,u,v) computes v=v+A(T) u
c*** n=# model params, m=# data
      parameter (mmod=50000,mxmat=20000000,mdat=300000)
      common/diwt/dwt(mmod)
      common/amat1/amat(mxmat)
      common/nmat1/nmat(mxmat)
      common/data/res(mdat),mmat(mdat)   
      dimension u(*),v(*)
      real*8 s(mmod)
      do i=1,n 
        s(i)=0.d0 
      enddo
      do i=1,m  
        do k=mmat(i),mmat(i+1)-1  
          s(nmat(k))=s(nmat(k))+amat(k)*u(i)
        enddo
      enddo
      do i=1,n
        v(i)=v(i)+s(i)*dwt(i)
      enddo
      return
      end

      subroutine smooth(ntot,flam,glam)
c*** construct smoother
      parameter (mdat=300000,mmod=50000,mxmat=20000000)
      common/amat1/amat(mxmat)
      common/nmat1/nmat(mxmat)
      common/diwt/dwt(mmod)
      common/data/res(mdat),mmat(mdat)
      common/blocks/r1(30),r2(30),hsz(180),bsz,
     +    nshell,mshell,mlat(181),nlat
 
      if(flam.le.0.) goto 400
c*** radial smoother -- first difference
      do 30 i=1,mshell
        ib=i
c        call iblk(r0,t0,p0,ib)
        do 30 j=2,nshell
c          rx=0.5*(r1(j)+r2(j))
c          call fblk(rx,t0,p0,jb)
c*** vertically adjacent blocks are spaced by mshell
          jb=ib+mshell
c  put in matrix
          ibase=mmat(ntot+1)
          amat(ibase)=flam
          nmat(ibase)=ib
          dwt(ib)=dwt(ib)+flam*flam
          amat(ibase+1)=-flam
          nmat(ibase+1)=jb
          dwt(jb)=dwt(jb)+flam*flam
          ntot=ntot+1
          res(ntot)=0.
          mmat(ntot+1)=mmat(ntot)+2
   30     ib=jb
c*** lateral smoother
  400 if(glam.le.0.) return
      do 40 j=1,nshell
         jmx=j*mshell
         jmn=jmx-mshell+1
         do 50 i=1,mshell
           ib=(j-1)*mshell+i
           call iblk(r0,t0,p0,ib)
c*** get N-S neighbors
           tx=t0+bsz
           px=p0
           if(tx.gt.180.) then
               tx=t0
               px=p0+180.
               if(px.gt.360.) px=px-360.
           end if
           call fblk(r0,tx,px,jbs)
           jbs=max(min(jbs,jmx),jmn)
           tx=t0-bsz
           px=p0
           if(tx.lt.0.) then
               tx=-tx
               px=p0+180.
               if(px.gt.360.) px=px-360.
           end if
           call fblk(r0,tx,px,jbn)
           jbn=max(min(jbn,jmx),jmn)

c*** get E-W neighbors
           it=t0/bsz+1
           px=p0-hsz(it)
           if(px.lt.0.) px=px+360.
           call fblk(r0,t0,px,jbw)
           jbw=max(min(jbw,jmx),jmn)
           px=p0+hsz(it)
           if(px.gt.360.) px=px-360.
           call fblk(r0,t0,px,jbe)
           jbe=max(min(jbe,jmx),jmn)

c  put in matrix
           ibase=mmat(ntot+1)
           amat(ibase)=4.*glam
           nmat(ibase)=ib
           dwt(ib)=dwt(ib)+16.*glam*glam
           amat(ibase+1)=-glam
           nmat(ibase+1)=jbn
           dwt(jbn)=dwt(jbn)+glam*glam
           amat(ibase+2)=-glam
           nmat(ibase+2)=jbs
           dwt(jbs)=dwt(jbs)+glam*glam
           amat(ibase+3)=-glam
           nmat(ibase+3)=jbe
           dwt(jbe)=dwt(jbe)+glam*glam
           amat(ibase+4)=-glam
           nmat(ibase+4)=jbw
           dwt(jbw)=dwt(jbw)+glam*glam
           ntot=ntot+1
           res(ntot)=0.
           mmat(ntot+1)=mmat(ntot)+5
   50      continue
   40   continue
      return
      end

      subroutine blks(nblk)
c*** routine to establish block parameters
      common/blocks/r1(30),r2(30),hsz(180),bsz,
     +    nshell,mshell,mlat(181),nlat
      character*256 filename
      data rad/57.29578/
      print *,'enter block descriptor file : '
      read(*,'(a)') filename
      open(12,file=filename)
      read(12,*) bsize
      nlat=nint(180./bsize)
      bsize=180./nlat
      bsz=bsize
      read(12,*) nshell
      ib=0
      do 10 n=1,nshell
        read(12,*) rad1,rad2
        tmax=0.
        r1(n)=rad1
        r2(n)=rad2
        mlat(1)=0
        do 30 ii=1,nlat
        tmin=tmax
        tmax=tmin+bsize
        th=0.5*(tmin+tmax)/rad
        s1=sin(th)
        mlon=max(nint(360./bsize*s1),1)
        hsize=360./mlon
        ib=ib+mlon
        if(n.eq.1) then
           hsz(ii)=hsize
           mlat(ii+1)=ib
        end if
   30   continue
      if(n.eq.1) mshell=ib
   10 continue
      nblk=ib
      close(12)
      return
      end


      subroutine fblk(r0,t0,p0,ib)
c*** routine to find block number given radial index and colat and long
      common/blocks/r1(30),r2(30),hsz(180),bsz,
     +    nshell,mshell,mlat(181),nlat
      do 10 i=1,nshell
      if(r0.ge.r1(i).and.r0.le.r2(i)) then
         n=i
         goto 20
      end if
   10 continue
      ib=-1
      return
   20 it=t0/bsz+1
      if(it.ge.1.and.it.le.nlat) then
        j1=(n-1)*mshell+mlat(it)+1
        ib=p0/hsz(it)+j1
      else
        ib=-1
      end if
      return
      end

      subroutine iblk(r0,t0,p0,ib)
c*** routine to find radius,colat and long of middle of block ib
      common/blocks/r1(30),r2(30),hsz(180),bsz,
     +    nshell,mshell,mlat(181),nlat
      kshell=float((ib-1))/float(mshell)+1
      r0=0.5*(r1(kshell)+r2(kshell))
      do 10 i=1,nlat
        j1=(kshell-1)*mshell+mlat(i)+1
        j2=(kshell-1)*mshell+mlat(i+1)
        if(ib.ge.j1.and.ib.le.j2) then
          t0=(i-0.5)*bsz
          p0=(ib-j1+0.5)*hsz(i)
          return
        end if
   10 continue
      r0=-1
      return
      end

      function gasdev(idum)
c returns normally distributed noise with zero mean and unit variance
c set idum to a neg integer to initialize
      save
      data iset/0/
      if(iset.eq.0) then
   5    v1=2.*ran1(idum)-1.
        v2=2.*ran1(idum)-1.
        r=v1*v1+v2*v2
        if(r.ge.1.) goto 5
        fac=sqrt(-2.*log(r)/r)
        gset=v1*fac
        gasdev=v2*fac
        iset=1
      else
        gasdev=gset
        iset=0
      endif
      return
      end

      function ran1(idum)
c returns random number with uniform prob between 0 and 1
c  set idum to any neg integer to initialize
      save
      common/rndom/r(97)
      parameter (m1=259200,ia1=7141,ic1=54773,rm1=1./m1)
      parameter (m2=134456,ia2=8121,ic2=28411,rm2=1./m2)
      parameter (m3=243000,ia3=4561,ic3=51349)
      data iff/0/
      if(idum.lt.0.or.iff.eq.0) then
        iff=1
        ix1=mod(ic1-idum,m1)
        ix1=mod(ia1*ix1+ic1,m1)
        ix2=mod(ix1,m2)
        ix1=mod(ia1*ix1+ic1,m1)
        ix3=mod(ix1,m3)
        do j=1,97
          ix1=mod(ia1*ix1+ic1,m1)
          ix2=mod(ia2*ix2+ic2,m2)
          r(j)=(float(ix1)+float(ix2)*rm2)*rm1
        enddo
        idum=1
      endif
      ix1=mod(ia1*ix1+ic1,m1)
      ix2=mod(ia2*ix2+ic2,m2)
      ix3=mod(ia3*ix3+ic3,m3)
      j=1+(97*ix3)/m3
      ran1=r(j)
      r(j)=(float(ix1)+float(ix2)*rm2)*rm1
      return
      end

